/*
 * Copyright 2011
 * 'MedCenter' is an interactive application for hospitals.
 * (c) Copyright 2011 - Abdelhafid Hafidi  <std.abdel[at]gmail.com>
 *                    - Alexandre Laurent  <lw.demoscene[at]gmail.com>
 *
 * [GPL]
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "MC_3DShapeModelList.h"

#include "MC_3DShapeModel.h"
#include "MC_3DShapeModelGL.h"
#include "MC_3DShapeModelGL2.h"

#include "MathUtils.h"

MC_3DShapeModelList :: MC_3DShapeModelList()
    :QObject(NULL),m_useVBO(false),
     m_minPoint(1000000,1000000,1000000),m_maxPoint(-1000000,-1000000,-1000000)
{

}

MC_3DShapeModelList :: ~MC_3DShapeModelList()
{
    this->clear();
}

void MC_3DShapeModelList :: clear()
{
    foreach (MC_3DShapeModel* pShapeModel, m_shapes)
    {
        delete pShapeModel;
    }

    m_minPoint = QVector3D(1000000,1000000,1000000);
    m_maxPoint = QVector3D(-1000000,-1000000,-1000000);

    m_shapes.clear();
    Q_EMIT changed();
}

bool MC_3DShapeModelList :: addShape(const QString& name, const MC_3DShape& shape)
{
    MC_3DShapeModel* pShapeModel;
    if ( m_useVBO )
    {
        pShapeModel = new MC_3DShapeModelGL2();
    }
    else
    {
        pShapeModel = new MC_3DShapeModelGL();
    }

    if ( pShapeModel == NULL )
    {
        qDebug() << "Fail to allocate memory for MC_3DShapeModel";
        return false;
    }

    if ( pShapeModel->loadShape(name,shape) == false )
    {
        return false;
    }

    m_shapes.push_back(pShapeModel);

    // Update min/max information
    m_minPoint.setX(qMin(shape.minPoint().x(), m_minPoint.x()));
    m_minPoint.setY(qMin(shape.minPoint().y(), m_minPoint.y()));
    m_minPoint.setZ(qMin(shape.minPoint().z(), m_minPoint.z()));

    m_maxPoint.setX(qMax(shape.maxPoint().x(), m_maxPoint.x()));
    m_maxPoint.setY(qMax(shape.maxPoint().y(), m_maxPoint.y()));
    m_maxPoint.setZ(qMax(shape.maxPoint().z(), m_maxPoint.z()));

    Q_EMIT changed();

    return true;
}

MC_3DShapeModel* MC_3DShapeModelList :: getShapeModel(unsigned int index)
{
    unsigned int nbShapes = m_shapes.count();
    if ( nbShapes == 0)
    {
        return NULL;
    }

    if ( index >= nbShapes )
    {
        index = nbShapes-1;
    }

    return m_shapes[index];
}

bool MC_3DShapeModelList :: contains(const QString& name)const
{
    foreach (MC_3DShapeModel* pShapeModel, m_shapes)
    {
        if ( name.contains(pShapeModel->name()))
        {
            return true;
        }
    }

    return false;
}

void MC_3DShapeModelList :: draw()
{
    foreach (MC_3DShapeModel* pShapeModel, m_shapes)
    {
        if ( pShapeModel->enabled() )
        {
            pShapeModel->draw();
        }
    }
}

QVector3D MC_3DShapeModelList :: center()const
{
    return ( m_minPoint + m_maxPoint ) / 2.0;
}

MC_3DShapeModel* MC_3DShapeModelList :: getClosestIntersectingShape(const QVector3D& rayStart, const QVector3D& rayEnd)const
{
    QVector3D ray = rayEnd - rayStart;
    // ray.normalize();
    QVector<MC_3DShapeModel*> listCollidingShaped;

    // For all shapes
    for ( QVector<MC_3DShapeModel*>::const_iterator itShapeModel = m_shapes.begin() ; itShapeModel != m_shapes.end() ; ++itShapeModel )
    {
        const MC_3DShape* pShapeData = (*itShapeModel)->get3DShape();

        QVector<QVector3D> vertices = pShapeData->getVertices();
        QVector<QVector3D> normals = pShapeData->getNormals();
        QVector<uint> indices = pShapeData->getIndices();

        // For all triangles
        for ( int i = 0 ; i < indices.size() ; i+=3 )
        {
            // First, get the vertices
            QVector3D v1 = vertices[indices[i]];
            QVector3D v2 = vertices[indices[i+1]];
            QVector3D v3 = vertices[indices[i+2]];

            // Since the normal may be kept as a interpolation of the triangles surrounding this triangle, we have to get back the original normal
            // If it was 3 times the same normal (as for one normal per triangles) nothing changed
            QVector3D normal = (normals[indices[i]] + normals[indices[i+1]] + normals[indices[i+2]]) / 3;

            float divFactor = QVector3D::dotProduct(ray,normal);
            if ( divFactor == 0.f )
            {
                qDebug() << "We are parallel";
                // The ray is // to the triangle ; Go next triangle
                continue;
            }

            qDebug() << "Ray: " << ray;
            float fract = (-1.f/divFactor) * ( QVector3D::dotProduct(rayStart,normal) * rayStart.distanceToPlane(v1,normal));
            qDebug() << "1/factor: " << (-1.f/divFactor);
            qDebug() << "Distance: " << rayStart.distanceToPlane(v1,normal);
            qDebug() << "dot(rayStart,normal)" << QVector3D::dotProduct(rayStart,normal) << " ; fract: " << fract;
            QVector3D collisionPoint = rayStart + fract * ray;

            qDebug() << "Collinding point: " << collisionPoint;

            // Now we have a point on the plane, but we still have to know if it is in the triangle or outside
            // To do that, we calculate the three dot product between the point and the three triangle vertices
            // If the sum of the angle is equal to 360, we are in.


            // Get the vectors to the point
            QVector3D pv1 = v1 - collisionPoint;
            pv1.normalize();
            QVector3D pv2 = v2 - collisionPoint;
            pv2.normalize();
            QVector3D pv3 = v3 - collisionPoint;
            pv3.normalize();

            float a1 = QVector3D::dotProduct(pv1,pv2);
            float a2 = QVector3D::dotProduct(pv2,pv3);
            float a3 = QVector3D::dotProduct(pv3,pv1);

            qDebug() << "Angles (deg): " << MathUtils::ranToDeg(acosf(a1)) << ";" << MathUtils::ranToDeg(acosf(a2)) << ";" << MathUtils::ranToDeg(acosf(a3));
            qDebug() << "Sum: " << acosf(a1) + acosf(a2) + acosf(a3);

            if ( acosf(a1) + acosf(a2) + acosf(a3) >= 6.28f )
            {
                qDebug() << "found";
                //listCollidingShaped.push_back(itShapeModel);
                break;
            }
        }
    }

    return NULL;
}

MC_3DShapeModel* MC_3DShapeModelList :: getClosestIntersectingShape2(const QVector3D& rayStart, const QVector3D& rayEnd)const
{
    QVector3D ray = rayEnd - rayStart;
    // ray.normalize();
    QVector<MC_3DShapeModel*> listCollidingShaped;

    // For all shapes
    for ( QVector<MC_3DShapeModel*>::const_iterator itShapeModel = m_shapes.begin() ; itShapeModel != m_shapes.end() ; ++itShapeModel )
    {
        const MC_3DShape* pShapeData = (*itShapeModel)->get3DShape();

        QVector<QVector3D> vertices = pShapeData->getVertices();
        QVector<QVector3D> normals = pShapeData->getNormals();
        QVector<uint> indices = pShapeData->getIndices();

        // For all triangles
        for ( int i = 0 ; i < indices.size() ; i+=3 )
        {
            // First, get the vertices
            QVector3D v1 = vertices[indices[i]];
            QVector3D v2 = vertices[indices[i+1]];
            QVector3D v3 = vertices[indices[i+2]];
            QVector3D triU = v2 - v1;
            QVector3D triV = v3 - v1;

            // Since the normal may be kept as a interpolation of the triangles surrounding this triangle, we have to get back the original normal
            // If it was 3 times the same normal (as for one normal per triangles) nothing changed
            QVector3D normal = QVector3D::crossProduct(triU,triV);
            normal.normalize();
            qDebug() << "Normal: " << normal;

            QVector3D rayDirection = rayEnd - rayStart;
            QVector3D w0 = rayStart - v1;

            float a = -QVector3D::dotProduct(normal,w0);
            float b = QVector3D::dotProduct(normal,rayDirection);
            if ( fabs(b) == 0.f )   // Ray parallel to plane
            {
                continue;
            }

            float r = a / b;
            if ( r < 0.0 )  // Ray goes away from triangle
            {
                continue;
            }

            QVector3D intersectionPoint = rayStart + r * rayDirection;
            qDebug() << "Intersection " << intersectionPoint;

            float uu = QVector3D::dotProduct(triU,triU);
            float uv = QVector3D::dotProduct(triU,triV);
            float vv = QVector3D::dotProduct(triV,triV);
            QVector3D w = intersectionPoint - v1;
            float wu = QVector3D::dotProduct(w,triU);
            float wv = QVector3D::dotProduct(w,triV);
            float d = uv * uv - uu * vv;

            float s, t;
            s = (uv * wv - vv * wu) / d;
            if (s < 0.0 || s > 1.0)        // I is outside T
            {
                continue;
            }

            t = (uv * wu - uu * wv) / d;
            if (t < 0.0 || (s + t) > 1.0)  // I is outside T
            {
                continue;
            }

            qDebug() << "Found";
        }
    }

    return NULL;
}
