#ifndef __ECGPARSERFACTORY_H__
#define __ECGPARSERFACTORY_H__

/*
 * Copyright 2011
 * 'MedCenter' is an interactive application for hospitals.
 * (c) Copyright 2011 - Abdelhafid Hafidi  <std.abdel[at]gmail.com>
 *                    - Alexandre Laurent  <lw.demoscene[at]gmail.com>
 *
 * [GPL]
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include <QMap>
#include <QString>

class QByteArray;

class ECGParser;
class ECGList;

class ECGParserFactory
{
private:

    QMap< QString , ECGParser* > m_parsers;

public:

    ~ECGParserFactory();

    void registerParser(const QString& extension, ECGParser* pParser);

    bool parse(const QString& fileName, ECGList& ecgList);
    bool parse(const QByteArray& data, const QString& dataType, ECGList& ecgList, const QString& ecgName);

    QStringList getAvailableExtensions()const;
};

/*! \class ECGParserFactory ECGParserFactory.h Parser/ECGParserFactory.h
    \brief Container for all parsers available

    The parsers can be registered using registerParser() and will be recognised by the parse() function.
    The parser function will call the correct parser depending on the extension of the file.
*/

/*! \fn ECGParserFactory::~ECGParserFactory();
 */

/*! \fn void ECGParserFactory::registerParser(const QString& extension, ECGParser* pParser);
    \brief Register the pParser in the factory
    \param extension the extension that will be associated with pParser.
    \param pParser the parser to register in the factory
    It is possible to pass an extension list where the separator is '|'.
    The QString will be decomposed and the parser will be registered for each members
 */

/*! \fn bool ECGParserFactory::parse(const QString& fileName, ECGList& ecgList);
    \brief parse the file corresponding to 'fileName' and fill the 'ecgList' with the data from the file
    \param fileName the name of the file to parse
    \param ecgList the list of ECGs where to put the new ECG
    \return true if the parse is done ; false if the file does not exist or the extension is not recognised
    The function checks if the file exist on the system.
    If the file exist, the extension (last characters after the last '.') are used to determine which parser to use.
 */

/*! \fn bool ECGParserFactory :: parse(const QByteArray& data, const QString& dataType, ECGList& ecgList, const QString& ecgName)
    \brief parse 'data' and fill the 'ecgList' with the data
    \param data the data to parser
    \param dataType the type of the data
    \param ecgList the list to fill
    \param ecgName the name of the new ECG
    \return true if the parse is done ; false if the data type is not known
 */

/*! \fn QStringList ECGParserFactory::getAvailableExtensions()const;
    \brief Go through the list of available parsers to get the extensions which can be parsed
    \return a QStringList containing the different extension possible to parse
*/

#endif
