/*
 * Copyright 2011
 * 'MedCenter' is an interactive application for hospitals.
 * (c) Copyright 2011 - Abdelhafid Hafidi  <std.abdel[at]gmail.com>
 *                    - Alexandre Laurent  <lw.demoscene[at]gmail.com>
 *
 * [GPL]
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "MedCenter.h"

#include <QPluginLoader>
#include <QDir>
#include <QCloseEvent>
#include <QMessageBox>

#include <cassert>

#include "Network/Client/NClient.h"
#include "Logger/FileLogger.h"

#include "meddatabase.hpp"

#include "UI/ConnectionQMLWidget.h"
#include "UI/MainScreenWidget.h"

#include "Types/Patient.h"
#include "Types/PatientData.h"
#include "Types/ListData.h"
#include "Types/OperationData.h"
#include "Types/FileData.h"
#include "Types/VersionData.h"
#include "Types/User.h"

#include "Loader.h"

#include "Utils/ByteArrayUtils.h"

#include "Module/ModuleInterface.h"

MedCenter :: MedCenter(QWidget* parent)
    :QWidget(parent)
{   
    m_pNetworkLogger = new FileLogger("./MedCenterServer.log");
    m_pNetworkManager = new NetworkManager(m_pNetworkLogger,this);
    m_pNetworkTimer = new QTimer(this);

    // Starting DB
    MedDatabase::initDatabase();

    connect(m_pNetworkManager,SIGNAL(newClientConnected(NClient*)),this,SLOT(sendAllData(NClient*)));
    connect(m_pNetworkManager,SIGNAL(machineListUpdated()),this,SLOT(updateMachinesList()));
    connect(m_pNetworkTimer,SIGNAL(timeout()),this,SLOT(processNetworkData()));
    m_pNetworkTimer->start(NETWORK_REFRESH_TIME);

    m_pConnectionWidget = new ConnectionQMLWidget(this);
    m_pConnectionWidget->load("qrc:/qml/QMLBox/Sources/ConnectScreen.qml");
    // m_pConnectionWidget->load("data/qml/ConnectScreen.qml");
    m_pConnectionWidget->show();
    connect(m_pConnectionWidget,SIGNAL(authentificationRequired(QString,QString)),this,SLOT(processUserConnection(QString,QString)));

    m_pMainScreenWidget = new MainScreenQMLWidget(&m_pluginsList,this);
    m_pMainScreenWidget->hide();
    connect(m_pMainScreenWidget,SIGNAL(requestPatient(QString)),this,SLOT(requestPatient(QString)));
    connect(this,SIGNAL(pluginsChanged()),m_pMainScreenWidget,SLOT(constructPlugins()));

    connect(m_pMainScreenWidget,SIGNAL(userNew(QString, QString, QString)),this,SLOT(userNew(QString, QString, QString)));
    connect(m_pMainScreenWidget,SIGNAL(userNameChanged(QString, QString)),this,SLOT(userNameChanged(QString,QString)));
    connect(m_pMainScreenWidget,SIGNAL(userCategoryChanged(QString, QString)),this,SLOT(userCategoryChanged(QString,QString)));
    connect(m_pMainScreenWidget,SIGNAL(deleteUser(QString)),this,SLOT(deleteUser(QString)));

    connect(m_pMainScreenWidget,SIGNAL(addMachine(QString, QString)),m_pNetworkManager,SLOT(addMachine(QString, QString)));
    connect(m_pMainScreenWidget,SIGNAL(deleteMachine(QString)),m_pNetworkManager,SLOT(deleteMachine(QString)));

    connect(m_pMainScreenWidget,SIGNAL(sendMessage(QString)),this,SLOT(sendMessage(QString)));

    connect(m_pMainScreenWidget,SIGNAL(patientChanged()),this,SLOT(patientUpdate()));

    // Loads some dummy data, just to show that the application works
    m_TMPPpatient = new Patient();
    {
        PatientData* pd = new PatientData("Dourouc",257);
        ListData<OperationData>* lop = new ListData<OperationData>();
        OperationData* op1 = new OperationData(-1,"Heart",QDate::currentDate(),"Small operation on heart");
        lop->add(op1);
        OperationData* op2 = new OperationData(-1,"Orteil",QDate(2001,6,12),"");
        lop->add(op2);

        ListData<FileData>* lfd1 = new ListData<FileData>();
        {
            lfd1->add(Loader::loadFile("data/3d/artery.trian","3DData","artery.trian","Heart"));
            lfd1->add(Loader::loadFile("data/3d/bone.trian","3DData","bone.trian","Heart"));
            lfd1->add(Loader::loadFile("data/3d/leftkidney.trian","3DData","leftkidney.trian","Heart"));
            lfd1->add(Loader::loadFile("data/3d/leftlung.trian","3DData","leftlung.trian","Heart"));
            lfd1->add(Loader::loadFile("data/3d/leftsurrenalgland.trian","3DData","leftsurrenalgland.trian","Heart"));
            lfd1->add(Loader::loadFile("data/3d/liver.trian","3DData","liver.trian","Heart"));
            lfd1->add(Loader::loadFile("data/3d/pancreas.trian","3DData","pancreas.trian","Heart"));
            lfd1->add(Loader::loadFile("data/3d/portalvein.trian","3DData","portalvein.trian","Heart"));
            lfd1->add(Loader::loadFile("data/3d/rightkidney.trian","3DData","rightkidney.trian","Heart"));
            lfd1->add(Loader::loadFile("data/3d/rightlung.trian","3DData","rightlung.trian","Heart"));
            lfd1->add(Loader::loadFile("data/3d/rightsurrenalgland.trian","3DData","rightsurrenalgland.trian","Heart"));
            lfd1->add(Loader::loadFile("data/3d/skin.trian","3DData","skin.trian","Heart"));
            lfd1->add(Loader::loadFile("data/3d/spleen.trian","3DData","spleen.trian","Heart"));
            lfd1->add(Loader::loadFile("data/3d/stomach.trian","3DData","stomach.trian","Heart"));
            lfd1->add(Loader::loadFile("data/3d/tumorleftsurregland.trian","3DData","tumorleftsurregland.trian","Heart"));
            lfd1->add(Loader::loadFile("data/3d/venoussystem.trian","3DData","venoussystem.trian","Heart"));
        }

        ListData<FileData>* lfd2 = new ListData<FileData>();
        {
            lfd2->add(Loader::loadFile("data/ecg/ecg.txt","ECGData","Heart signal","Heart"));
        }

        VersionData* pVD = new VersionData();

        m_TMPPpatient->add("PatientData",pd);
        m_TMPPpatient->add("Operations",lop);
        m_TMPPpatient->add("3DData",lfd1);
        m_TMPPpatient->add("ECGData",lfd2);
        m_TMPPpatient->add("VersionData",pVD);

        m_patients.add(m_TMPPpatient);
    }

    this->loadPlugins();
}

MedCenter :: ~MedCenter()
{
    delete m_pConnectionWidget;
    delete m_pMainScreenWidget;

    delete m_pNetworkManager;
    delete m_pNetworkLogger;

    this->clearPlugins();

    MedDatabase::closeDatabase();
}

void MedCenter :: setScreen(QWidget* pScreen)
{
    assert(pScreen);

    m_pConnectionWidget->hide();
    m_pMainScreenWidget->hide();

    pScreen->show();
    qDebug() << "MC Size " << pScreen->size();
    // this->setMinimumSize(pScreen->minimumSize());
    this->resize(pScreen->size());
}

bool MedCenter :: loadPlugin(QObject* pPlugin)
{
    if ( pPlugin != NULL )
    {
        ModuleInterface* pModule = qobject_cast<ModuleInterface*>(pPlugin);
        if ( pModule )
        {
            QString name = pModule->name();
            m_pluginsList[name] = pModule;

            return true;
        }

        qDebug() << "Plugin object type not recognised";
        return false;
    }

    return true;
}

void MedCenter :: clearPlugins()
{
    foreach(ModuleInterface* pModule, m_pluginsList)
    {
        delete pModule;
    }
    m_pluginsList.clear();
}

bool MedCenter :: loadPlugins()
{
    bool bResult = true;

    // static plugins loading
    foreach(QObject* pPlugin, QPluginLoader::staticInstances())
    {
        bResult &= loadPlugin(pPlugin);
    }

    // additionnal plugins loading (in ./plugins folder)
    QDir pluginsDir("./plugins");
    foreach (QString fileName, pluginsDir.entryList(QDir::Files))
    {
        qDebug() << "Plugin file: '" << fileName << "'";
        QPluginLoader pluginLoader(pluginsDir.absoluteFilePath(fileName));
        QObject* pPlugin = pluginLoader.instance();
        bResult &= loadPlugin(pPlugin);
    }

    emit pluginsChanged();
    return bResult;
}


void MedCenter :: sendAllData(NClient* pClient)
{
    assert(pClient);

    QList<Patient*> patients = m_patients.getPatientsList();
    foreach(Patient* pPatient, patients)
    {
        m_pNetworkManager->write(pClient,NetworkManager::MT_PatientInfo,pPatient->data());
    }
}

void MedCenter :: processNetworkData()
{
    const QList<QByteArray>& data = m_pNetworkManager->dataReceived();
    bool updated = false;
    foreach(QByteArray dataPacket, data)
    {
        // First dispatching
        int nbReadType = 0;
        NetworkManager::MessageType dataType = static_cast<NetworkManager::MessageType>(ByteArrayUtils::readNumber(dataPacket,0,&nbReadType));
        if (nbReadType == -1)
        {
            qDebug() << "Fail to read the type of data for Network packet";
            continue;
        }

        switch ( dataType )
        {
            case NetworkManager::MT_ChatMessage:
            {
                QString message = ByteArrayUtils::readString(dataPacket,nbReadType,NULL);
                m_pMainScreenWidget->addMessage(message);
            }
            break;
            case NetworkManager::MT_PatientInfo:
            {
                Patient* pPatient = new Patient();
                pPatient->load(dataPacket,nbReadType);
                if ( m_patients.add(pPatient) == false )
                {
                    delete pPatient;
                }
                else
                {
                    updated = true;

                    PatientData* pPatientData = dynamic_cast<PatientData*>(pPatient->get("PatientData"));
                    if ( m_lastRequestedPatient == pPatientData->name() )
                    {
                        QMessageBox::information(this,tr("Update"),tr("The patient information has been updated"));
                        this->requestPatient(m_lastRequestedPatient);
                    }
                }
            }
            break;
            default:
                break;
        }
    }

    if ( updated )
    {
        m_pMainScreenWidget->setPatientNames(m_patients.getPatientNameList());
    }

    m_pNetworkManager->clearData();
    m_pNetworkTimer->start();
}

void MedCenter :: processUserConnection(QString userName, QString password)
{
    qDebug() << "Connection: " << userName << " Pass: " << password;

    if ( MedDatabase::connectUser(userName,password) )
    {
        this->setScreen(m_pMainScreenWidget);
        m_pMainScreenWidget->setPatientNames(m_patients.getPatientNameList());

        m_connectedUser = MedDatabase::user(userName);
        m_pMainScreenWidget->setRightLevel(m_connectedUser.category());
        if ( m_connectedUser.category() >= User::UC_Admin )
        {
            m_pMainScreenWidget->setUsersList(MedDatabase::listUsers());
            m_pMainScreenWidget->setMachinesList(m_pNetworkManager->getMachinesList());
        }
    }
    else
    {
        QMessageBox::warning(this,"Invalid User","You entered invalid identifiants");
    }
}

void MedCenter :: disconnectUser()
{
    MedDatabase::disconnectUser(m_connectedUser.name());

    m_pConnectionWidget->clear();
    this->setScreen(m_pConnectionWidget);
}

void MedCenter :: requestPatient(QString patientName)
{
    m_lastRequestedPatient = patientName;
    m_pMainScreenWidget->setPatient(m_patients.get(patientName));
}

void MedCenter :: resizeEvent(QResizeEvent *pResizeEvent)
{
    if ( m_pConnectionWidget->isVisible() )
    {
        m_pConnectionWidget->resize(pResizeEvent->size());
    }

    if ( m_pMainScreenWidget->isVisible() )
    {
        m_pMainScreenWidget->resize(pResizeEvent->size());
    }
}

void MedCenter :: closeEvent(QCloseEvent *pCloseEvent)
{
    if ( m_pMainScreenWidget->isVisible() )
    {
        this->disconnectUser();
        pCloseEvent->ignore();
    }
}

void MedCenter :: userNew(QString name, QString password, QString category)
{
    if ( !name.isEmpty() && !password.isEmpty() && !category.isEmpty())
    {
        MedDatabase::addUser(name, password, category);

        m_pMainScreenWidget->setUsersList(MedDatabase::listUsers());
    }
}


void MedCenter :: deleteUser(QString userName)
{
    MedDatabase::deleteUser(userName);

    m_pMainScreenWidget->setUsersList(MedDatabase::listUsers());
}

void MedCenter :: userNameChanged(QString oldName, QString newName)
{
    MedDatabase::changeUserName(oldName,newName);

    m_pMainScreenWidget->setUsersList(MedDatabase::listUsers());
}

void MedCenter :: userCategoryChanged(QString name, QString newCategory)
{
    MedDatabase::changeUserCategory(name,newCategory);

    m_pMainScreenWidget->setUsersList(MedDatabase::listUsers());
}

void MedCenter :: updateMachinesList()
{
    m_pMainScreenWidget->setMachinesList(m_pNetworkManager->getMachinesList());
}

void MedCenter :: sendMessage(QString newMessage)
{
    QByteArray data;
    QString message = m_connectedUser.name() + ": " + newMessage;
    ByteArrayUtils::writeString(data,message);

    m_pNetworkManager->write(NetworkManager::MT_ChatMessage,data);

    m_pMainScreenWidget->addMessage(message);
}

void MedCenter :: patientUpdate()
{
    m_pNetworkManager->write(NetworkManager::MT_PatientInfo,m_patients.get(m_lastRequestedPatient)->data());
    this->requestPatient(m_lastRequestedPatient);
}
