#ifndef __MEDCENTER_H__
#define __MEDCENTER_H__

/*
 * Copyright 2011
 * 'MedCenter' is an interactive application for hospitals.
 * (c) Copyright 2011 - Abdelhafid Hafidi  <std.abdel[at]gmail.com>
 *                    - Alexandre Laurent  <lw.demoscene[at]gmail.com>
 *
 * [GPL]
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include <QWidget>
#include <QTimer>
#include <QMap>

#include "Network/NetworkManager.h"
#include "PatientContainer.h"
#include "Types/User.h"

class Logger;
class ConnectionQMLWidget;
class MainScreenQMLWidget;
class ModuleInterface;

class MedCenter : public QWidget
{
    Q_OBJECT

    static const int NETWORK_REFRESH_TIME = 1000;   /*!< Delay between each connection test */

private:

    PatientContainer m_patients;                    /*!< */
    Patient* m_TMPPpatient;                         /*!< */
    QString m_lastRequestedPatient;                 /*!< */

    NetworkManager* m_pNetworkManager;              /*!< */
    Logger* m_pNetworkLogger;                       /*!< */
    QTimer* m_pNetworkTimer;                        /*!< */

    User m_connectedUser;                           /*!< */

    ConnectionQMLWidget* m_pConnectionWidget;       /*!< */
    MainScreenQMLWidget* m_pMainScreenWidget;       /*!< */

    QMap<QString, ModuleInterface*> m_pluginsList;  /*!< */

    void setScreen(QWidget* pScreen);
    bool loadPlugin(QObject* pPlugin);

protected:

    void resizeEvent(QResizeEvent* pResizeEvent);
    void closeEvent(QCloseEvent *pCloseEvent);

public:
    MedCenter(QWidget* parent=0);
    ~MedCenter();

    void clearPlugins();
    bool loadPlugins();

    Q_SIGNAL void pluginsChanged();

    Q_SLOT void sendAllData(NClient* pClient);
    Q_SLOT void processNetworkData();

    Q_SLOT void processUserConnection(QString userName, QString password);
    Q_SLOT void disconnectUser();
    Q_SLOT void requestPatient(QString patientName);

    Q_SLOT void userNew(QString name, QString password, QString category);
    Q_SLOT void deleteUser(QString userName);
    Q_SLOT void userNameChanged(QString oldName, QString newName);
    Q_SLOT void userCategoryChanged(QString name, QString newCategory);

    Q_SLOT void updateMachinesList();

    Q_SLOT void sendMessage(QString newMessage);

    Q_SLOT void patientUpdate();
};

/*! \class MedCenter MedCenter.h "MedCenter.h"
    \brief Base of the application
 */

/*! \fn MedCenter::MedCenter(QWidget* parent=0)
    \param parent
 */

/*! \fn MedCenter::~MedCenter()
 */

/*! \fn void MedCenter::setScreen(QWidget* pScreen);
    \brief Sets the screen to display
    \param pScreen pointer to the screen to display
 */

/*! \fn void MedCenter::loadPlugin(QObject* pPlugin);
    \brief try to load the plugin
    \param pPlugin pointer to the plugin to load
 */

/*! \fn void MedCenter::resizeEvent(QResizeEvent* pResizeEvent);
    \brief Transfers the resize event to the underlying interfaces
    \param pResizeEvent
 */

/*! \fn void MedCenter::closeEvent(QCloseEvent *pCloseEvent);
    \brief When closing from the data information, returns to the connect screen
    \param pCloseEvent
 */

/*! \fn MedCenter::MedCenter(QWidget* parent=0);
    \brief
    \param parent
 */

/*! \fn MedCenter::~MedCenter();
 */

/*! \fn void MedCenter::clearPlugins();
    \brief removes all plugins
 */

/*! \fn bool MedCenter::loadPlugins();
    \brief loads all plugins
 */

/*! \fn void MedCenter::pluginsChanged();
    \brief Signal send when a plugin is newly loaded
*/

/*! \fn void MedCenter::sendAllData(NClient* pClient);
    \brief Send all data contained on this machine to one client (the new connected to this machine)
    \param pClient
 */

/*! \fn void MedCenter::processNetworkData();
    \brief Function parsing the network data
 */

/*! \fn void MedCenter::processUserConnection(QString userName, QString password);
    \brief Connect a user to the application
    \param userName
    \param password
 */

/*! \fn void MedCenter::disconnectUser();
    \brief disconnects
 */

/*! \fn void MedCenter::requestPatient(QString patientName);
    \brief request for information on a patient
    \param patientName
 */

/*! \fn void MedCenter::userNew(QString name, QString password, QString category);
    \brief create a new user
    \param name
    \param password
    \param category
 */

/*! \fn void MedCenter::deleteUser(QString userName);
    \brief delete a user
    \param userName
 */

/*! \fn void MedCenter::userNameChanged(QString oldName, QString newName);
    \brief Changes the name of a user
    \param oldName
    \param newName
 */

/*! \fn void MedCenter::userCategoryChanged(QString name, QString newCategory);
    \brief changes the category for a user
    \param name
    \param newCategory
 */

/*! \fn void MedCenter::updateMachinesList();
    \brief Updates the list of machines to connect
 */

/*! \fn void MedCenter::sendMessage(QString newMessage);
    \brief Send a chat message over the network
    \param newMessage
 */

#endif
