/*
 * Copyright 2011
 * 'MedCenter' is an interactive application for hospitals.
 * (c) Copyright 2011 - Abdelhafid Hafidi  <std.abdel[at]gmail.com>
 *                    - Alexandre Laurent  <lw.demoscene[at]gmail.com>
 *
 * [GPL]
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "SSLServer.h"

#include <QSslSocket>
#include <QSslKey>
#include <QFile>

#include <QDebug>

#include "Network/SSLException.h"

#include "Logger/Logger.h"

SSLServer :: SSLServer(quint16 port, Logger* pLogger, QObject* parent)
    :NServer(port,pLogger,parent)
{
    if ( !QFile::exists("./data/SSL_Cert/server-key.pem") )
    {
        m_pLogger->warning("File 'server-key.pem' not found");
        throw SSLException();
    }

    if ( !QFile::exists("./data/SSL_Cert/ca.pem") )
    {
        m_pLogger->warning("File 'ca.pem' not found");
        throw SSLException();
    }

    if ( !QFile::exists("./data/SSL_Cert/server-crt.pem") )
    {
        m_pLogger->warning("File 'server-crt.pem' not found");
        throw SSLException();
    }
}

void SSLServer :: reportErrors(QList<QSslError> sslErrors)
{
    foreach(QSslError sslError, sslErrors)
    {
        qDebug() << "SSLErrors: " << sslError.errorString();
    }
}

/*
 * Function coming from the documentation, to secure newly connected sockets
 */
void SSLServer::incomingConnection(int socketDescriptor)
{
    qDebug() << "IncomingConnection (" << socketDescriptor << ")";
    QSslSocket *pServerSocket = new QSslSocket();

    if (pServerSocket->setSocketDescriptor(socketDescriptor))
    {
        connect(pServerSocket, SIGNAL(sslErrors(QList<QSslError>)), this, SLOT(reportErrors(QList<QSslError>)));
        connect(pServerSocket,SIGNAL(error(QAbstractSocket::SocketError)), this, SLOT(reportError(QAbstractSocket::SocketError)));

        connect(pServerSocket, SIGNAL(readyRead()), this, SLOT(appendData()));
        connect(pServerSocket, SIGNAL(disconnected()), this, SLOT(cleanUp()));

        QFile file( "./data/SSL_Cert/server-key.pem" );
        if( ! file.open( QIODevice::ReadOnly ) )
        {
            m_pLogger->warning("Failed to open the private key for server");
            delete pServerSocket;
            return;
        }

        QSslKey key( &file, QSsl::Rsa, QSsl::Pem, QSsl::PrivateKey, "MedCenterSSL");
        file.close();

        pServerSocket->setPrivateKey( key );

        if( ! pServerSocket->addCaCertificates( "./data/SSL_Cert/ca.pem" ) )
        {
            m_pLogger->warning("Fail to open './data/SSL_Cert/ca.pem'");
            delete pServerSocket;
            return;
        }

        pServerSocket->setLocalCertificate("./data/SSL_Cert/server-crt.pem");

        // Client verification removed (we are just encoding the connexion)
        // That's should not be done, but since we have fake certificate / fake client too
        pServerSocket->setPeerVerifyMode(QSslSocket::VerifyNone);

        // Ignoring errors since we have auto signed certificate
        pServerSocket->ignoreSslErrors();

        pServerSocket->startServerEncryption();

        m_sockets.append(new NServerSocket(pServerSocket));
    }
    else
    {
        delete pServerSocket;
    }
}
