/*
 * Copyright 2011
 * 'MedCenter' is an interactive application for hospitals.
 * (c) Copyright 2011 - Abdelhafid Hafidi  <std.abdel[at]gmail.com>
 *                    - Alexandre Laurent  <lw.demoscene[at]gmail.com>
 *
 * [GPL]
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "QMLEditDataWidget.h"

#include <QDeclarativeContext>
#include <QDeclarativeView>
#include <QDeclarativeComponent>

#include <QFile>
#include <QFileInfo>
#include <QFileDialog>

#include "Types/Patient.h"
#include "Types/PatientData.h"
#include "Types/VersionData.h"
#include "Types/FileData.h"
#include "Types/ListData.h"
#include "Types/OperationData.h"

#include "UI/Types/FileDataObject.h"

QMLEditDataWidget :: QMLEditDataWidget(QWidget *parent)
    :QMLWidget(parent),m_pPatient(NULL)
{
    // qmlRegisterType<FileDataObject>("FileDataObject", 1,0, "FileDataObject");
}

bool QMLEditDataWidget :: updateQML()
{
    if ( m_pQMLObject )
    {
        return true;
    }

    return NULL;
}

void QMLEditDataWidget :: sendPatientData()
{
    PatientData* pPatientData = dynamic_cast<PatientData*>(m_pPatient->get("PatientData"));
    if ( pPatientData )
    {
        QList<QObject*> tmpList;
        QObject* pObject = dynamic_cast<QObject*>(pPatientData);
        tmpList.append(pObject);
        m_pView->rootContext()->setContextProperty("patientData", QVariant::fromValue(tmpList));
    }

    ListData<OperationData>* pListOperations = dynamic_cast<ListData<OperationData> *>(m_pPatient->get("Operations"));
    if ( pListOperations )
    {
        QList<QObject*> tmpList;
        foreach (OperationData* pOperation, pListOperations->dataList())
        {
            tmpList.append(pOperation);
        }
        m_pView->rootContext()->setContextProperty("patientOperationData", QVariant::fromValue(tmpList));
    }

    m_filesList.clear();
    QStringList fileNames = m_pPatient->getFileFolderList();
    foreach (QString names, fileNames )
    {
        ListData<FileData>* pListFiles = dynamic_cast<ListData<FileData>* >(m_pPatient->get(names));
        if ( pListFiles )
        {
            foreach(FileData* pFile, pListFiles->dataList())
            {
                FileDataObject* pTmpFileObject = new FileDataObject(names,pFile->folder(),pFile->name());
                m_filesList.add(pTmpFileObject);
            }
        }
    }
    m_pView->rootContext()->setContextProperty("fileDataModel", QVariant::fromValue(m_filesList.getList()));
}

void QMLEditDataWidget :: createPatient()
{
    m_pPatient = new Patient();
    if ( m_pPatient == NULL )
    {
        qDebug() << "Fail to allocate memory for Patient";
    }
    else
    {
        PatientData* pPD = new PatientData();
        ListData<OperationData>* pLOP = new ListData<OperationData>();
        VersionData* pVD = new VersionData();
        pVD->addChangement(Changement("User",QDateTime::currentDateTime(),"Creation of the patient"));

        m_pPatient->add("PatientData",pPD);
        m_pPatient->add("Operations",pLOP);
        m_pPatient->add("VersionData",pVD);

        emit newPatient(m_pPatient);
    }
}

void QMLEditDataWidget :: setPatient(Patient* pPatient)
{
    assert(pPatient);

    if ( pPatient == m_pPatient )
    {
        this->sendPatientData();
        return;
    }
    else
    {
        m_pPatient = pPatient;
        this->sendPatientData();
    }

    // Slot connected only here, to avoid the notification of change caused by setting the data
    connect(m_pQMLObject,SIGNAL(patientNameChanged(QString)),this,SLOT(setPatientName(QString)));
    connect(m_pQMLObject,SIGNAL(patientAgeChanged(QString)),this,SLOT(setPatientAge(QString)));

    connect(m_pQMLObject,SIGNAL(operationNameChanged(QString,QString)),this,SLOT(setOperationName(QString,QString)));
    connect(m_pQMLObject,SIGNAL(operationDateChanged(QString,QString)),this,SLOT(setOperationDate(QString,QString)));
    connect(m_pQMLObject,SIGNAL(operationObservationChanged(QString,QString)),this,SLOT(setOperationObservation(QString,QString)));
    connect(m_pQMLObject,SIGNAL(operationNew()),this,SLOT(addOperation()));

    connect(m_pQMLObject,SIGNAL(fileNew(QString, QString)),this,SLOT(createNewFile(QString, QString)));
    connect(m_pQMLObject,SIGNAL(fileCategoryChanged(QString,QString,QString,QString)),this,SLOT(setFileCategory(QString,QString,QString,QString)));
    connect(m_pQMLObject,SIGNAL(fileFolderChanged(QString,QString,QString,QString)),this,SLOT(setFileFolder(QString,QString,QString,QString)));
    connect(m_pQMLObject,SIGNAL(fileRemoved(QString,QString,QString)),this,SLOT(removeFile(QString,QString,QString)));
}

void QMLEditDataWidget :: setPatientName(QString name)
{
    PatientData* pPD = dynamic_cast<PatientData*>(m_pPatient->get("PatientData"));
    if ( pPD->name() == name )
    {
        return;
    }

    pPD->setName(name);

    VersionData* pVD = dynamic_cast<VersionData*>(m_pPatient->get("VersionData"));
    pVD->addChangement(Changement("User",QDateTime::currentDateTime(),"Patient name edited"));
    emit patientChanged();
}

void QMLEditDataWidget :: setPatientAge(QString age)
{
    int newAge = age.toInt();

    PatientData* pPD = dynamic_cast<PatientData*>(m_pPatient->get("PatientData"));
    if ( pPD->age() == newAge )
    {
        return;
    }

    pPD->setAge(newAge);

    VersionData* pVD = dynamic_cast<VersionData*>(m_pPatient->get("VersionData"));
    pVD->addChangement(Changement("User",QDateTime::currentDateTime(),"Patient age edited"));
    emit patientChanged();
}

void QMLEditDataWidget :: addOperation()
{
    ListData<OperationData>* pLOP = dynamic_cast<ListData<OperationData>*>(m_pPatient->get("Operations"));
    OperationData* pOperationData = new OperationData();

    // Here we avoid to have multiple times the same name
    // The loop checks if the name that we want to assign is already existing or not
    QString newOperationName = "New operation";
    int i = 1;
    while ( OperationData::searchName(pLOP->dataList(),newOperationName) != NULL)
    {
        newOperationName = "New operation " + QString::number(i);
        i++;
    }

    pOperationData->setName(newOperationName);

    pLOP->add(pOperationData);

    VersionData* pVD = dynamic_cast<VersionData*>(m_pPatient->get("VersionData"));
    pVD->addChangement(Changement("User",QDateTime::currentDateTime(),"Added new operation"));

    emit patientChanged();
}

void QMLEditDataWidget :: setOperationName(QString oldOperationName, QString newOperationName)
{
    if ( oldOperationName == newOperationName )
    {
        return;
    }

    ListData<OperationData>* pLOP = dynamic_cast<ListData<OperationData>*>(m_pPatient->get("Operations"));
    OperationData* pOperationData = OperationData::searchName(pLOP->dataList(),oldOperationName);
    pOperationData->setName(newOperationName);

    VersionData* pVD = dynamic_cast<VersionData*>(m_pPatient->get("VersionData"));
    pVD->addChangement(Changement("User",QDateTime::currentDateTime(),"Changed operation name"));

    emit patientChanged();
}

void QMLEditDataWidget :: setOperationDate(QString operationName, QString operationDate)
{
    QDate newDate = QDate::fromString(operationDate);
    if ( !newDate.isValid() )
    {
        this->sendPatientData();
        return;
    }

    ListData<OperationData>* pLOP = dynamic_cast<ListData<OperationData>*>(m_pPatient->get("Operations"));
    OperationData* pOperationData = OperationData::searchName(pLOP->dataList(),operationName);
    if ( pOperationData->date() == newDate )
    {
        return;
    }

    pOperationData->setDate(newDate);

    VersionData* pVD = dynamic_cast<VersionData*>(m_pPatient->get("VersionData"));
    pVD->addChangement(Changement("User",QDateTime::currentDateTime(),"Changed operation date"));

    emit patientChanged();
}

void QMLEditDataWidget :: setOperationObservation(QString operationName, QString observation)
{
    ListData<OperationData>* pLOP = dynamic_cast<ListData<OperationData>*>(m_pPatient->get("Operations"));
    OperationData* pOperationData = OperationData::searchName(pLOP->dataList(),operationName);
    if ( pOperationData->observation() == observation )
    {
        return;
    }

    pOperationData->setObservation(observation);

    VersionData* pVD = dynamic_cast<VersionData*>(m_pPatient->get("VersionData"));
    pVD->addChangement(Changement("User",QDateTime::currentDateTime(),"Changed operation observation"));

    emit patientChanged();
}

void QMLEditDataWidget :: createNewFile(QString category, QString folder)
{
    QString filters = tr("Resources files (*.*)");

    QStringList fileNames = QFileDialog::getOpenFileNames(this,tr("Open 3D file"),QDir::currentPath(),filters);
    if ( !fileNames.isEmpty() )
    {
        ListData<FileData>* pNewList = NULL;
        if ( !fileNames.contains(category) )
        {
            // The category does not exist. Making it!
            pNewList = new ListData<FileData>();
            m_pPatient->add(category,pNewList);
        }
        else
        {
            pNewList = dynamic_cast<ListData<FileData>* >(m_pPatient->get(category));
        }

        foreach(QString fileName, fileNames)
        {
            if ( QFile::exists(fileName) )
            {
                QFile dataFile(fileName);
                if ( dataFile.open(QIODevice::ReadOnly) )
                {
                    pNewList->add(new FileData(category,QFileInfo(dataFile).fileName(),folder,dataFile.readAll()));
                }

                dataFile.close();
            }
        }

        this->sendPatientData();
        emit patientChanged();
    }
}

void QMLEditDataWidget :: setFileCategory(QString oldCategory, QString newCategory, QString folderName, QString fileName)
{
    if ( oldCategory == newCategory )
    {
        return;
    }

    QStringList fileNames = m_pPatient->getFileFolderList();
    if ( !fileNames.contains(oldCategory) )
    {
        qDebug() << "Old category not found";
        return;
    }

    ListData<FileData>* pNewList = NULL;
    if ( !fileNames.contains(newCategory) )
    {
        // The category does not exist. Making it!
        pNewList = new ListData<FileData>();
        m_pPatient->add(newCategory,pNewList);
    }
    else
    {
        pNewList = dynamic_cast<ListData<FileData>* >(m_pPatient->get(newCategory));
    }

    // We need to search in the old list, the correct file
    ListData<FileData>* pOldList = dynamic_cast<ListData<FileData>* >(m_pPatient->get(oldCategory));
    foreach (FileData* pFileData, pOldList->dataList())
    {
        if ( pFileData->folder() == folderName && pFileData->name() == fileName)
        {
            pOldList->remove(pFileData);
            pNewList->add(pFileData);
        }
    }

    this->sendPatientData();
    emit patientChanged();
}

void QMLEditDataWidget :: setFileFolder(QString category, QString oldFolder, QString newFolder, QString fileName)
{
    if ( oldFolder == newFolder )
    {
        return;
    }

    QStringList fileNames = m_pPatient->getFileFolderList();
    if ( !fileNames.contains(category) )
    {
        qDebug() << "Category not found";
        return;
    }

    ListData<FileData>* pList = dynamic_cast<ListData<FileData>* >(m_pPatient->get(category));
    foreach (FileData* pFileData, pList->dataList())
    {
        if ( pFileData->folder() == oldFolder && pFileData->name() == fileName)
        {
            pFileData->setFolder(newFolder);
        }
    }

    this->sendPatientData();
    emit patientChanged();
}

void QMLEditDataWidget :: removeFile(QString category, QString folderName, QString fileName)
{
    QStringList fileNames = m_pPatient->getFileFolderList();
    if ( !fileNames.contains(category) )
    {
        qDebug() << "Category not found";
        return;
    }

    // We need to search in the old list, the correct file
    ListData<FileData>* pList = dynamic_cast<ListData<FileData>* >(m_pPatient->get(category));
    foreach (FileData* pFileData, pList->dataList())
    {
        if ( pFileData->folder() == folderName && pFileData->name() == fileName)
        {
            pList->remove(pFileData);
            delete pFileData;
        }
    }

    this->sendPatientData();
    emit patientChanged();
}

