/*
 * Copyright 2011
 * 'MedCenter' is an interactive application for hospitals.
 * (c) Copyright 2011 - Abdelhafid Hafidi  <std.abdel[at]gmail.com>
 *                    - Alexandre Laurent  <lw.demoscene[at]gmail.com>
 *
 * [GPL]
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "RadioMap.h"

#include <QString>
#include <QByteArray>

#include <QDebug>

#include <cmath>

bool RadioMap :: isExisting(const QString& name)
{
    foreach(Radio r, m_radios)
    {
        if ( r.name() == name )
        {
            return true;
        }
    }

    return false;
}

bool RadioMap :: loadRadio(const QString& fileName)
{
    if ( this->isExisting(fileName) )
    {
        return true;
    }

    Radio newRadio = Radio::load(fileName);
    if ( newRadio.radio().isNull() )
    {
        qDebug() << "Fail to load radio '" << fileName << "'";
        return false;
    }

    m_radios.append(newRadio);

    return true;
}

bool RadioMap :: loadRadio(const QString& name, const QByteArray& data)
{
    if ( this->isExisting(name) )
    {
        return true;
    }

    Radio newRadio = Radio::load(name,data);
    if ( newRadio.radio().isNull() )
    {
        qDebug() << "Fail to load radio '" << name << "'";
        return false;
    }

    m_radios.append(newRadio);

    return true;
}

const QPixmap* RadioMap :: getMasterRadio()
{
    if ( m_radios.size() == 0 )
    {
        return NULL;
    }

    return &m_radios.at(0).radio();
}

QPixmap RadioMap :: getMasterRadio(float zoomFactor)
{
    if ( m_radios.size() == 0 )
    {
        return QPixmap();
    }

    if ( zoomFactor == 1 || zoomFactor == 0 )
    {
        return *this->getMasterRadio();
    }

    QSize radioSize = m_radios.at(0).radio().size();
    QSize scaledSize = radioSize * zoomFactor;

    return m_radios.at(0).radio().scaled(scaledSize);
}

const QPixmap* RadioMap :: getRadio(unsigned int x, unsigned int y)
{
    if ( m_radios.size() == 0 )
    {
        return NULL;
    }

    if ( x > this->width() )
    {
        return NULL;
    }

    if ( y > this->height() )
    {
        return NULL;
    }

    unsigned int index = y * this->width() + x + 1;
    return &m_radios.at(index).radio();
}

QPixmap RadioMap :: getRadio(const QPoint& radioIndex, float zoomFactor)
{
    if ( radioIndex.x() < 0 && radioIndex.y() < 0 )
    {
        return getMasterRadio(zoomFactor);
    }

    if ( m_radios.size() == 0 )
    {
        return QPixmap();
    }

    if ( static_cast<unsigned int>(radioIndex.x()) > this->width() )
    {
        return QPixmap();
    }

    if ( static_cast<unsigned int>(radioIndex.y()) > this->height() )
    {
        return QPixmap();
    }

    unsigned int index = radioIndex.y() * this->width() + radioIndex.x() + 1;

    QSize radioSize = m_radios.at(index).radio().size();
    QSize scaledSize = radioSize * zoomFactor;
    return m_radios.at(index).radio().scaled(scaledSize);
}

unsigned int RadioMap :: width()const
{
    if ( m_radios.size() == 0 )
    {
        return 0;
    }

    unsigned int nbRadios = m_radios.size() - 1;
    return sqrt(nbRadios);
}

unsigned int RadioMap :: height()const
{
    if ( m_radios.size() == 0 )
    {
        return 0;
    }

    unsigned int nbRadios = m_radios.size() - 1;
    return sqrt(nbRadios);
}
